/*
 * check3.h
 *
 * 3 state check box control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: check3.h,v $
 * Revision 1.19  2001/05/22 12:49:32  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.18  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.17  1999/03/09 08:01:47  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.16  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.15  1998/09/23 06:22:59  robertj
 * Added open source copyright license.
 *
 * Revision 1.14  1995/04/25 10:50:55  robertj
 * Fixed warning on hiding virtuals.
 *
 * Revision 1.13  1995/03/14 12:41:04  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.12  1994/11/28  12:31:45  robertj
 * Documentation.
 *
 * Revision 1.11  1994/10/30  11:46:26  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.10  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.9  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.8  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.7  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.6  1994/01/15  02:50:29  robertj
 * Moved control contructors to common and added a platform dependent Construct() function.
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PCHECK3WAYBOX

#ifdef __GNUC__
#pragma interface
#endif



/**A 3 state check box control that has {\it Checked}, {\it Unchecked} and
   {\it Neither} states.
   
   The exact appearence of a check box is platform dependent, however, they
   all have some visual distinction between the three states plus a string
   label associated with it.
 */
class PCheck3WayBox : public PNamedControl
{
  PCLASSINFO(PCheck3WayBox, PNamedControl);

  public:
   /**The three possible states for the 3 way check box.
     */
    enum CheckValues {
      /// State indicating OFF.
      Unchecked,  
      /// State indicating ON.
      Checked,    
      /// State indicating unknown.
      Neither     
    };

   /**Create a 3 way check box control with the specified name next to it,
       callback function and the initial check state.
     */
    PCheck3WayBox(
      PInteractor * parent,  /// Interactor into which the control is placed.
      CheckValues value = Unchecked   /// Initial state for the check box.
    );
    PCheck3WayBox(
      PInteractor * parent,  /// Interactor into which the control is placed.
      const PString & name,  /// Name to be placed next to the check box.
      CheckValues value = Unchecked   /// Initial state for the check box.
    );
    PCheck3WayBox(
      PInteractor * parent,  /// Interactor into which the control is placed.
      const PString & name,  /// Name to be placed next to the check box.
      const PNotifier & notify,   /// Function to call when changes state.
      CheckValues value = Unchecked   /// Initial state for the check box.
    );

   /**Create control from interactor layout with the specified control ID.
       This is mainly used in support of resource based dialogs.
     */
    PCheck3WayBox(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      CheckValues * valuePtr      /// Variable to change to the check box state.
    );

    /** Destroy the 3 way check box. */
    virtual ~PCheck3WayBox();


  /**@name Overrides from class PControl */
   /**This function transfers the value of the control to or from the variable
       pointed to by the value pointer member variable.
     */
    virtual void TransferValue(
      int option 
     /**Transfer value option. When this is -1 when the function transfers
         the value from the value pointer into the control. This is called in
         \Ref{PDialog::OnInit()} function. When option is zero then the
         function transfers the value from the control to the value pointer
         variable. This is called just before the callback function every time
         the check box state changes.
       */
    );


  /**@name New functions for class */
   /**Set the default dimensions for the control. For the width this is the
       width of the name part plus 3 times the average width of a character.
       For the height this is 1.5 times the height of the font.
     */
    void DefaultDimensions();

   /**Get the current state of the check box.

       @return
       check box state.
     */
    CheckValues GetValue() const;

   /**Set the check state of the check box. This will be updated on the
       screen immediately (within OS contsraints).
     */
    void SetValue(
      CheckValues newVal  /// New state to set the check box to.
    );

   /**Get the current value pointer associated with the control. The variable
       pointed to by this is autamatically updated with the current value of
       the check box.

       @return
       value pointer associated with the control.
     */
    CheckValues * GetValuePointer() const;

   /**Set the current value pointer associated with the control. The variable
       pointed to by this is autamatically updated with the current value of
       the check box.
     */
    void SetValuePointer(
      CheckValues * ptr  /// New value pointer to associate with the control.
    );


  private:
  // New functions for class
    // Common constructor code used by platform dependent code.
    void Construct();


// Include platform dependent part of class
#include <pwlib/check3.h>
};


// End Of File ///////////////////////////////////////////////////////////////
