/*
 * cursor.h
 *
 * Cursor description.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: cursor.h,v $
 * Revision 1.15  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.14  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.13  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.12  1998/09/23 06:23:15  robertj
 * Added open source copyright license.
 *
 * Revision 1.11  1995/03/14 12:41:16  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.10  1994/12/21  11:52:49  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.9  1994/12/13  11:47:49  robertj
 * Documentation.
 *
 * Revision 1.8  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.7  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.6  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PCURSOR

#ifdef __GNUC__
#pragma interface
#endif


/**This class represents the specification for a cursor that moves in tandem
   with the mouse or other pointing device.

   An instance of a cursor is attached to every \Ref{PInteractor} class and is
   automatically made current whenever the mouse is over that interactor.

   A cursor consists of a small pixel image that interacts with the screen to
   yield transparent parts and or inverted parts as well as fixed colour
   image parts.
 */
class PCursor : public PContainer
{
  PCONTAINERINFO(PCursor, PContainer);

  public:
    /** Create a standard arrow cursor. */
    PCursor();

   /**Create a cursor as loaded from the resources for the application. The
       resource identifier is a unique number associated with the cursor in
       the resources.
     */
    PCursor(
      PRESOURCE_ID resID    /// Resource identifier constant.
    );

   /**Create a cursor from the raw pixel images. The dimensions of the pixel
       images must be the same as determined by the
       \Ref{PApplication::GetCursorSize()} function. They must also have a
       depth of 1 bit per pixel.
       
       Pixels on the screen are determined by applying the and mask to it, then
       the xor mask. Thus pixels can be transparent, inverted, cleared or set
       as required.
       
       The hot spot is the poin within the cursor bit map that is the actual
       cursor position, eg the point of the arrow cursor, the centre of a
       cross hair cursor etc.
     */
    PCursor(
      const PPixelImage & andMask,  /// Mask for pixels in the cursor.
      const PPixelImage & xorMask,  /// Value for pixels in the cursor.
      const PPoint & hotSpot        /// Hot spot relative to corner of bit map
    );


  /**@name Overrides from class PObject */
   /**Determine if the two cursors are the same. Note that this is different
       from having two cursors that merely {\bf look} the same. They must be
       references to the same cursor or if loaded from resources, the same
       resource ID.
       
       @return
       #EqualTo# if reference the same cursor,
       #GreaterThan# if different.
     */
    virtual Comparison Compare(
      const PObject & obj   /// Other cursor to compare against.
    ) const;


  private:
    // Overrides from class PContainer
    virtual BOOL SetSize(PINDEX newSize);


// Include platform dependent part of class
#include <pwlib/cursor.h>
};


// End Of File ///////////////////////////////////////////////////////////////
