/*
 * dcanvas.h
 *
 * Drawing canvas for interactors.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: dcanvas.h,v $
 * Revision 1.15  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.14  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.13  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.12  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.11  1998/09/23 06:23:17  robertj
 * Added open source copyright license.
 *
 * Revision 1.10  1995/03/14 12:41:17  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.9  1994/12/13  11:47:51  robertj
 * Documentation.
 *
 * Revision 1.8  1994/10/23  04:42:21  robertj
 * Added capability to over draw child windows when creating canvas.
 *
 * Revision 1.7  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.6  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.5  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PDRAWCANVAS

#ifdef __GNUC__
#pragma interface
#endif


/**A canvas that is attached to an interactor on the screen and may be drawn
   into at any time. Unlike a \Ref{PRedrawCanvas} which the user should never
   directly create, the PDrawCanvas is spcifically for application driven
   drawing of the screen.

  *********************************** NOTE ***********************************
  On Some platforms, canvases are a limited resource. Do NOT create and leave
  them around for long periods. As soon as it is not required till the next
  user event, destroy it!
  ****************************************************************************
 */
class PDrawCanvas : public PInteractorCanvas
{
  PCLASSINFO(PDrawCanvas, PInteractorCanvas);

  public:
    /** Create a canvas associated with an interactor on the screen. */
    PDrawCanvas(
      PInteractor * theInteractor,  /// Interactor canvas is attached to.
      BOOL inPixels = FALSE,
     /**Set the initial canvas coordinates are in screen pixels. The default
         is for the coordinates to be set to one based on the current font for
         the interactor (1/4 the average character widthand 1/8 the height).
       */
      BOOL overDrawChildWindows = FALSE
     /**Drawing operations may overwrite child interactors that the attached
         interactor may have conatined within it. The default is for the
         drawing clip region to be set so that drawing does not overwrite
         child interactors.
       */
    );

    /** Destroy the canvas and its resources. */
    virtual ~PDrawCanvas();


// Include platform dependent part of class
#include <pwlib/dcanvas.h>
};


// End Of File ///////////////////////////////////////////////////////////////
