/*
 * icon.h
 *
 * Standard icon class.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: icon.h,v $
 * Revision 1.17  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.16  1999/03/10 03:49:51  robertj
 * More documentation adjustments.
 *
 * Revision 1.15  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.14  1998/09/23 06:23:48  robertj
 * Added open source copyright license.
 *
 * Revision 1.13  1995/03/14 12:41:31  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.12  1995/01/10  11:42:58  robertj
 * Stuffed up constructor name.
 *
 * Revision 1.11  1994/12/21  11:53:04  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.10  1994/09/25  10:38:28  robertj
 * Moved GetDimensions() to platform dependent code.
 *
 * Revision 1.9  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.8  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.7  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.6  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.5  1993/12/31  06:42:49  robertj
 * Made inlines optional for debugging purposes.
 * Added ImgIcon class.
 *
 * Revision 1.4  1993/12/01  16:09:05  robertj
 * Windows NT port.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PICON

#ifdef __GNUC__
#pragma interface
#endif


/**This class represents the sepcification for a platform specific icon. This
   is typically used to indicate the presence of some modal object in the
   user interface. For instance, an instance of an icon is attached to every
   \Ref{PTitledWindow} class and is used whenever the that interactor is made
   iconic.

   An icon consists of a small pixel image that interacts with the screen to
   yield transparent parts and or inverted parts as well as fixed colour
   image parts.
 */
class PIcon : public PImgIcon
{
  PCONTAINERINFO(PIcon, PImgIcon);

  public:
    /** Create icon. */
    PIcon(
      PRESOURCE_ID resID    /// Resource identifier constant.
    );
      
   /**Create a cursor from the raw pixel images. The dimensions of the pixel
       images must be the same as determined by the
       \Ref{PApplication::GetIconSize()} function. The and mask must also have
       a depth of 1 bit per pixel. If the xor mask is also 1 bit per pixel then
       it operates as a logical xor on data. If it is a colour pix map then
       the actual colour is placed at that location.

       Pixels on the screen are determined by applying the and mask to it, then
       the xor mask. Thus pixels can be transparent, inverted, cleared or set
       as required.
     */
    PIcon(
      const PPixelImage & andMask,  /// Mask for pixels in the icon.
      const PPixelImage & xorMask   /// Value for pixels in the icon.
    );

    /**@name Overrides from class PObject */
   /**Determine if the two icons are the same. Note that this is different
       from having two icons that merely {\bf look} the same. They must be
       references to the same icon or if loaded from resources, the same
       resource ID.

       @return
       #EqualTo# if reference the same icon,
       #GreaterThan# if different.
     */
    virtual Comparison Compare(
      const PObject & obj   /// Other icon to compare against.
    ) const;


// Include platform dependent part of class
#include <pwlib/icon.h>
};


// End Of File ///////////////////////////////////////////////////////////////
