/*
 * mailgui.h
 *
 * Electronic Mail GUI interface.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: mailgui.h,v $
 * Revision 1.8  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.7  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.6  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.5  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.4  1998/11/30 03:01:00  robertj
 * New directory structure
 *
 * Revision 1.3  1998/09/23 06:24:09  robertj
 * Added open source copyright license.
 *
 * Revision 1.2  1995/08/12 22:29:43  robertj
 * Rework of GUI mail class.
 *
 * Revision 1.1  1995/04/01 08:24:14  robertj
 * Initial revision
 *
 */

#define _PMAILGUI

#ifdef __GNUC__
#pragma interface
#endif


#include <ptlib/mail.h>


/**This class establishes a mail session with the platforms mail system using
   a Graphical User Interface.
*/
class PMailGUI : public PMail
{
  PCLASSINFO(PMailGUI, PMail);

  public:
   /**Create a mail session using a Graphical User Interface. If the
       #parent# parameter is non-null then a dialog is presented
       to log the user onto the mail system.

       Note that not all platforms may display a dialog at all. The user may
       be implicitly logged into the mail system as in Unix or logged in via
       another process as in MS-Windows when another mail program is open.
     */
    PMailGUI(
      PInteractor * parent = NULL    /// Parent window for GUI interface.
    );


   /**Destroy the mail session, logging off the mail system if necessary.
     */
    virtual ~PMailGUI();


  /**@name New functions for class */
   /**Attempt to log on to the mail system using a Graphical User Interface.
       A dialog may be presented to allow the user to select a user name and
       password.

       If the #parent# parameter is NULL then the owner interactor
       will be the application main window.

       Note that not all platforms may display a dialog at all. The user may
       be implicitly logged into the mail system as in Unix or logged in via
       another process as in MS-Windows when another mail program is open.

       @return
       TRUE if successfully logged on.
     */
    BOOL LogOnGUI(
      PInteractor * parent = NULL    /// Parent window for GUI interface.
    );

   /**Display a system standard dialog for sending a mail message. All of the
       mail actions: entering text, addresses, attachments etc, are performed
       automatically by the displayed GUI.

       @return
       TRUE if successfully logged on.
     */
    BOOL SendGUI();

   /**Get the interactor thatr owns the mail session dialogs.

       @return
       Pointer to owner interactor.
     */
    PInteractor * GetOwner() const
      { return owner; }


  protected:
    /** Owner window of mail GUI windows */
    PInteractor * owner;


// Include platform dependent part of class
#include <pwlib/mailgui.h>
};


// End Of File ///////////////////////////////////////////////////////////////
