/*
 * mdidoc.h
 *
 * Multiple Document Interface Document Window.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: mdidoc.h,v $
 * Revision 1.16  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.15  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.14  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.13  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.12  1998/09/23 06:24:13  robertj
 * Added open source copyright license.
 *
 * Revision 1.11  1995/03/14 12:41:43  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.10  1995/01/03  09:36:12  robertj
 * Documentation.
 *
 * Revision 1.9  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.8  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.7  1994/07/21  23:49:31  robertj
 * Removed redundent destructors.
 *
 * Revision 1.6  1994/07/17  10:46:06  robertj
 * Fixed up MDI document window title.
 *
 * Revision 1.5  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PMDIDOCWINDOW

#ifdef __GNUC__
#pragma interface
#endif


/**This interactor defines a Multiple Document Interface child window. The
   document windows are platform dependent in their exact appearance and their
   relationship with the \Ref{PMDIFrameWindow} that owns them.
 */
class PMDIDocWindow : public PTitledWindow
{
  PCLASSINFO(PMDIDocWindow, PTitledWindow);

  public:
   /**Construct a new MDI child document window. The MDI child window has the
       same basic capabilities as a \Ref{PTopLevelWindow} as defines by the
       common ancestor. These define the attributes of a full overlapped
       window.
     */
    PMDIDocWindow(
      PMDIFrameWindow *parent,  /// Owner MDI frame window.
      const PString & title,    /// Initial title for child window.
      int CanDo=CanClose|CanGrow|CanIconify|CanZoom  /// Titled window flags.
    );


  /**@name Overrides from class PTitledWindow */
   /**Set the new window title string. This will be updated on the screen
       immediately (subject to OS constraints). It will also update the window
       list menu to reflect the new window title.
     */
    virtual void SetTitle(
      const PString & title   /// New title for MDI child window.
    );



// Include platform dependent part of class
#include <pwlib/mdidoc.h>
};


// End Of File ///////////////////////////////////////////////////////////////
