/*
 * meditbox.h,
 *
 * Multi-line edit box.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: meditbox.h,v $
 * Revision 1.18  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.17  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.16  1999/03/09 08:01:48  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.15  1999/02/16 08:08:45  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.14  1998/09/23 06:24:17  robertj
 * Added open source copyright license.
 *
 * Revision 1.13  1995/06/04 08:49:10  robertj
 * Removed redundent destructor.
 *
 * Revision 1.12  1995/04/22 00:39:16  robertj
 * Added common construct code.
 *
 * Revision 1.11  1995/03/14 12:41:45  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.10  1995/01/03  09:36:14  robertj
 * Documentation.
 *
 * Revision 1.9  1994/10/30  11:46:49  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.8  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.7  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.6  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.5  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.4  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */

#define _PMULTILINEEDITBOX

#ifdef __GNUC__
#pragma interface
#endif

/**This control is a text editing box that allows multiple lines to be entered.
   The usual capabilities for editing text are available, though the exact
   features are platform dependent.
 */
class PMultiLineEditBox : public PEditBox
{
  PCLASSINFO(PMultiLineEditBox, PEditBox);

  public:
    /** Create a multi-line edit box control. */
    PMultiLineEditBox(
      PInteractor * parent  /// Interactor into which the control is placed.
    );

    /** Create control from interactor layout with the specified control ID. */
    PMultiLineEditBox(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      PString * valuePtr          /// Variable to change to the editor value.
    );


  /**@name New functions for class */
   /**Get the text of the line specified in the control. If the line number
       requested is beyond all lines in the control, then an empty string is
       returned.

       @return
       string containing the specified line.
     */
    PINDEX GetLineCount();
    /** Return the number of lines of text in the edit box. */

    PString GetLine(
      PINDEX linenum   /// Line number to get.
    );

   /**Determine the line number (first line zero) given an index into the
       character array buffer of all lines in the edit box.
       
       @return
       line number for character offset.
     */
    PINDEX GetLineFromOffset(
      PINDEX offset   /// Character offset into text to find line number.
    );

   /**Determine the offset index into the character array buffer of all the
       text in the edit box that is the start of the specified line number.
       
       @return
       offset for line number.
     */
    PINDEX GetOffsetFromLine(
      PINDEX linenum   /// Line number to find offset of.
    );


  private:
    void Construct();
      // Common contructor code.


// Include platform dependent part of class
#include <pwlib/meditbox.h>
};


// End Of File ///////////////////////////////////////////////////////////////
