/*
 * menusep.h
 *
 * Menu Separator class.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: menusep.h,v $
 * Revision 1.14  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.13  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.12  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.11  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.10  1998/09/23 06:24:23  robertj
 * Added open source copyright license.
 *
 * Revision 1.9  1995/03/14 12:41:48  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.8  1995/01/03  09:36:17  robertj
 * Documentation.
 *
 * Revision 1.7  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.6  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.5  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.4  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.3  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PMENUSEPARATOR

#ifdef __GNUC__
#pragma interface
#endif


/**A class representing a menu item separator. This may be inserted into a
   menu to place a platform dependent separator between menu items.
 */
class PMenuSeparator : public PMenuEntry
{
  PCLASSINFO(PMenuSeparator, PMenuEntry);

  friend class PSubMenu;

  public:
    /** Create a single line menu separator as a menu entry in the menu. */
    PMenuSeparator(
      PSubMenu & menu,      /// Menu into which the new entry is to be placed.
      PMenuEntry * before = NULL
     /**Menu entry before which the entry is to be inserted. If this is NULL
         then the menu entry is appended to the end of the menu.
       */
    );


  /**@name New functions for class */
    /** Set the string name of the menu entry. This function does nothing. */
    virtual void SetString(
      const PString & str   /// New string for the menu entry.
    );

   /**Get the current string name of the menu entry. 

       @return
       empty string.
     */
    virtual PString GetString() const;


// Include platform dependent part of class
#include <pwlib/menusep.h>
};


// End Of File ///////////////////////////////////////////////////////////////
