/*
 * control.h
 *
 * Control interactor ancestor class.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: control.h,v $
 * Revision 1.13  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.12  1998/11/30 02:59:53  robertj
 * New directory structure
 *
 * Revision 1.11  1998/09/24 03:39:48  robertj
 * Added open software license.
 *
 * Revision 1.10  1998/09/21 13:30:48  robertj
 * Changes to support new PListView class. Different call back method.
 *
 * Revision 1.9  1996/08/08 10:09:36  robertj
 * Directory structure changes for common files.
 *
 * Revision 1.8  1995/03/12 05:00:12  robertj
 * Re-organisation of DOS/WIN16 and WIN32 platforms to maximise common code.
 * Used built-in equate for WIN32 API (_WIN32).
 *
 * Revision 1.7  1995/01/22  07:29:44  robertj
 * Added font & colour standard dialogs.
 *
 * Revision 1.6  1994/10/30  11:47:30  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.5  1994/10/23  05:45:21  robertj
 * Changed window messaging technique.
 *
 * Revision 1.4  1994/07/17  10:46:06  robertj
 * Enhancements, bug fixes etc.
 *
 * Revision 1.3  1994/06/25  11:55:15  robertj
 * Unix version synchronisation.
 *
 * Revision 1.2  1994/04/20  12:17:44  robertj
 * assert changes
 *
 * Revision 1.1  1994/04/12  08:21:52  robertj
 * Initial revision
 *
 */

#ifndef _PCONTROL

///////////////////////////////////////////////////////////////////////////////
// PControl

#define _PCONTROL_PLATFORM_INCLUDE
#include "../../control.h"

#endif
#ifdef _PCONTROL_PLATFORM_INCLUDE
#undef _PCONTROL_PLATFORM_INCLUDE

  public:
    // Overrides from class PInteractor
    virtual void CreateHWND();
      // Create the MS-Windows handle for the interactor.

    virtual int TranslateOption(NMHDR & msg) const;
      // Translate the windows notification message code to the PWLib
      // notify function code. This returns -1 if the windows message is to be
      // ignored.

  protected:
    virtual void DefWndProc();
      // All controls have MS-Windows messages intercepted and passed through
      // this function.


  private:
    void SubClassControlWndProc();
    // Function to subclass the controls WndProc.

    // Member variables
    WNDPROC wndProc;
      // The default control WndProc, the control is subclassed to allow the
      // PWLib system to intercept messages that it is interested in.

#endif


// End Of File ///////////////////////////////////////////////////////////////
