/*
 * numedbox.h
 *
 * Named control class
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: numedbox.h,v $
 * Revision 1.18  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.17  1999/03/10 03:49:52  robertj
 * More documentation adjustments.
 *
 * Revision 1.16  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.15  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.14  1998/09/23 06:24:29  robertj
 * Added open source copyright license.
 *
 * Revision 1.13  1995/03/14 12:41:52  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.12  1994/12/21  11:53:16  robertj
 * Documentation and variable normalisation.
 *
 * Revision 1.11  1994/10/30  11:46:54  robertj
 * Changed mechanism for doing notification callback functions.
 *
 * Revision 1.10  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.9  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.8  1994/03/07  07:38:19  robertj
 * Major enhancementsacross the board.
 *
 * Revision 1.7  1994/01/03  04:42:23  robertj
 * Mass changes to common container classes and interactors etc etc etc.
 *
 * Revision 1.6  1993/09/27  16:35:25  robertj
 * Removed special constructor for dialog resource loading.
 *
 * Revision 1.5  1993/08/24  00:27:59  robertj
 * Reimplemented nudge buttons for new push button arrangement.
 *
 * Revision 1.4  1993/08/21  01:50:33  robertj
 * Made Clone() function optional, default will assert if called.
 *
 * Revision 1.3  1993/08/21  01:07:35  robertj
 * Added nudge buttons
 *
 * Revision 1.2  1993/07/14  12:49:16  robertj
 * Fixed RCS keywords.
 *
 */


#define _PNUMBEREDITBOX

#ifdef __GNUC__
#pragma interface
#endif


/**A text editing box that only allows numbers to be entered. This usually
   will have next to the editable text box two "nudge buttons" which allow the
   value of the number edit box to incremented or decremented by a preset
   amount.
   
   This is an abstract class for any form of numeric input. There are two main
   descendents with PWLib: \Ref{PIntegerEditBox} and \Ref{PFloatEditBox}
   classes for editing integers and floating point numbers respectively.
 */
class PNumberEditBox : public PEditBox
{
  PCLASSINFO(PNumberEditBox, PEditBox);

  public:
    /** Create a new number edit box. */
    PNumberEditBox(
      PInteractor * parent  /// Interactor into which the control is placed.
    );

    /** Create control from interactor layout with the specified control ID. */
    PNumberEditBox(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      void * valuePtr             /// Variable to change to the editor value.
    );

    /** Destroy the number edit box and its nudge buttons. */
    virtual ~PNumberEditBox();


  protected:
  /**@name New functions for class */
    /** Add the nudge value to the edit box value. */
    virtual void AddNudge() = 0;

    /** Subtract the nudge value to the edit box value. */
    virtual void SubtractNudge() = 0;


    // Member fields
    class NudgeButton : public PPushButton
    {
      PCLASSINFO(NudgeButton, PPushButton)
      public:
        NudgeButton(PNumberEditBox * parent, BOOL isUp);

        // Overrides from class PInteractor
        virtual void OnDrawFace(PCanvas & canvas,
                           const PRect & rect, BOOL hasFocus, BOOL isSelected);
          // Draw the contents of the nudge button.

      private:
        BOOL up;
    };
    NudgeButton * nudgeUp;
    NudgeButton * nudgeDown;


    PDECLARE_NOTIFIER(NudgeButton, PNumberEditBox, Nudged);
    // Call back from nudge buttons.


// Include platform dependent part of class
#include <pwlib/numedbox.h>
};


// End Of File ///////////////////////////////////////////////////////////////
