/*
 * prjobdlg.h
 *
 * Print job dialog.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: prjobdlg.h,v $
 * Revision 1.12  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.11  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.10  1999/03/09 08:01:49  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.9  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.8  1998/09/23 06:28:26  robertj
 * Added open source copyright license.
 *
 * Revision 1.7  1995/06/17 11:13:04  robertj
 * Documentation update.
 *
 * Revision 1.6  1995/03/14 12:42:13  robertj
 * Updated documentation to use HTML codes.
 *
 * Revision 1.5  1995/01/11  09:45:08  robertj
 * Documentation and normalisation.
 *
 * Revision 1.4  1994/08/23  11:32:52  robertj
 * Oops
 *
 * Revision 1.3  1994/08/22  00:46:48  robertj
 * Added pragma fro GNU C++ compiler.
 *
 * Revision 1.2  1994/04/11  14:15:38  robertj
 * Normalised dialog resoruce id constants.
 *
 * Revision 1.1  1994/04/01  14:26:22  robertj
 * Initial revision
 *
 */


#define _PPRINTJOBDIALOG

#ifdef __GNUC__
#pragma interface
#endif


/**A system specific standard dialog for getting print job information. This
   dialog is displayed just before starting a print.

   Note that unlike most dialogs, most operations such as
   \Ref{PInteractor::Show()}, \Ref{PInteractorLayout::GetControl()} etc may
   {\bf not} be executed before the \Ref{PModalDialog::RunModal()} function
   has been executed. Furthermore, they are not available {\bf after}
   \Ref{PModalDialog::RunModal()} has returned. This is due to the
   implementation on some platforms which goes to great lengths to use the
   actual standard file dialog for the platform.
 */
class PPrintJobDialog : public PPrintDialog
{
  PCLASSINFO(PPrintJobDialog, PPrintDialog);

  public:
   /**Create the standard print job dialog. The second form of constructor
       preinitialises the fields in the dialog to that specified in the
       printer information class. Otherwise a system default will be used.
     */
    PPrintJobDialog(
      PInteractor * parent,    /// Owner interactor for the dialog.
      PRESOURCE_ID resID = PSTD_ID_DIALOG_PRINT_JOB
        /// Resource identifier for the dialog description resource.
    );
    PPrintJobDialog(
      PInteractor * parent,    /// Owner interactor for the dialog.
      const PPrintInfo & info, /// Initial setup printer information.
      PRESOURCE_ID resID = PSTD_ID_DIALOG_PRINT_JOB
        /// Resource identifier for the dialog description resource.
    );

    /** Destroy the print job dialog. */
    ~PPrintJobDialog();


  private:
  // New members for class
    void Construct();
    // Constructor code common to all constrcutors.


// Include platform dependent part of class
#include <pwlib/prjobdlg.h>
};


// End Of File ///////////////////////////////////////////////////////////////
