/*
 * statrect.h
 *
 * Static rectangle control.
 *
 * Portable Windows Library
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: statrect.h,v $
 * Revision 1.6  2001/05/22 12:49:33  robertj
 * Did some seriously wierd rewrite of platform headers to eliminate the
 *   stupid GNU compiler warning about braces not matching.
 *
 * Revision 1.5  1999/03/10 03:49:53  robertj
 * More documentation adjustments.
 *
 * Revision 1.4  1999/03/09 08:01:50  robertj
 * Changed comments for doc++ support (more to come).
 *
 * Revision 1.3  1999/02/16 08:08:46  robertj
 * MSVC 6.0 compatibility changes.
 *
 * Revision 1.2  1998/09/23 06:29:21  robertj
 * Added open source copyright license.
 *
 * Revision 1.1  1995/12/10 13:04:46  robertj
 * Initial revision
 *
 */


#define _PSTATICRECT

#ifdef __GNUC__
#pragma interface
#endif


/**This control displays a filled rectangle. Used to visually decorate
   dialogs.
   
   This control is display only and normally does not permit any further
   interaction.
   
   The exact apperance of this control is platform dependent, but they all
   serve the same purpose and have the same semantics.
 */
class PStaticRect : public PControl
{
  PCLASSINFO(PStaticRect, PControl);

  public:
   /**Create a filled rectangle control.
     */
    PStaticRect(
      PInteractor * parent    /// Interactor into which the control is placed
    );
    PStaticRect(
      PInteractor * parent,   /// Interactor into which the control is placed
      const PColour & colour  /// Colour for the filled rectangle.
    );

    /** Create control from interactor layout with the specified control ID. */
    PStaticRect(
      PInteractorLayout * parent, /// Interactor into which the box is placed.
      PRESOURCE_ID ctlID,         /// Identifier for the control in the layout.
      const PNotifier & notify,   /// Function to call when changes state.
      void * valuePtr             /// Unused value pointer for static control.
    );


// Include platform dependent part of class
#include <pwlib/statrect.h>
};


// End Of File ///////////////////////////////////////////////////////////////
